@@ -1,7 +1,7 @@
 /*
  * IPVS         Application module
  *
- * Version:     $Id: ip_vs_app.c,v 1.6 2001/01/10 07:42:53 wensong Exp $
+ * Version:     $Id: ip_vs_app.c,v 1.7 2001/02/23 09:15:05 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -16,8 +16,6 @@
  *
  *		IP_MASQ_APP application masquerading module
  *
- * 	$Id: ip_vs_app.c,v 1.6 2001/01/10 07:42:53 wensong Exp $
- *
  * Author:	Juan Jose Ciarlante, <jjciarla@raiz.uncu.edu.ar>
  *
  */
@@ -263,7 +261,9 @@ vs_fix_ack_seq(const struct ip_vs_seq *vseq, struct tcphdr *th)
          * for packets before most recent resized pkt, use previous_delta
          */
         if (vseq->delta || vseq->previous_delta) {
-                if(after(ack_seq, vseq->init_seq)) {
+                /* since ack_seq is the number of octet that is expected
+                   to receive next, so compare it with init_seq+delta */
+                if(after(ack_seq, vseq->init_seq+vseq->delta)) {
                         th->ack_seq = htonl(ack_seq - vseq->delta);
                         IP_VS_DBG(9, "vs_fix_ack_seq(): subtracted delta "
                                   "(%d) from ack_seq\n", vseq->delta);
