@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.13 2001/01/29 15:27:27 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.14 2001/02/23 09:15:05 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -188,7 +188,7 @@ ip_vs_sched_persist(struct ip_vs_service *svc, struct iphdr *iph)
 	/*
 	 * As far as we know, FTP is a very complicated network protocol, and
 	 * it uses control connection and data connections. For active FTP,
-	 * FTP server initilize data connection to the client, its source port
+	 * FTP server initialize data connection to the client, its source port
 	 * is often 20. For passive FTP, FTP server tells the clients the port
 	 * that it passively listens to,  and the client issues the data
 	 * connection. In the tunneling or direct routing mode, the load
@@ -337,7 +337,7 @@ ip_vs_sched_persist(struct ip_vs_service *svc, struct iphdr *iph)
 	}
 
 	/*
-	 *    Bind the connection entry with the vs dest.
+	 *    Bind the connection entry with its destination.
 	 */
 	ip_vs_bind_dest(cp, dest);
 
@@ -1246,7 +1246,8 @@ static int __init ip_vs_init(void)
 
 static void __exit ip_vs_cleanup(void)
 {
-	nf_unregister_hook(&ip_vs_post_routing_ops);
+	nf_unregister_hook(&ip_vs_forward_icmp_ops);
+        nf_unregister_hook(&ip_vs_post_routing_ops);
 	nf_unregister_hook(&ip_vs_out_ops);
 	nf_unregister_hook(&ip_vs_in_ops);
 	ip_vs_app_cleanup();
