@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.11 2001/01/29 15:27:27 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.13 2001/02/15 15:17:59 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -718,12 +718,12 @@ static int ip_vs_add_dest(struct ip_vs_service *svc,
 	dest = ip_vs_get_trash_dest(svc, daddr, dport);
 	if (dest != NULL) {
 		IP_VS_DBG(3, "Get destination %u.%u.%u.%u:%u from trash, "
-			"refcnt=%d, service %lu/%u.%u.%u.%u:%u\n",
-			NIPQUAD(daddr), ntohs(dport),
-			atomic_read(&dest->refcnt),
-			dest->vfwmark,
-			NIPQUAD(dest->vaddr),
-			ntohs(dest->vport));
+                          "refcnt=%d, service %lu/%u.%u.%u.%u:%u\n",
+                          NIPQUAD(daddr), ntohs(dport),
+                          atomic_read(&dest->refcnt),
+                          dest->vfwmark,
+                          NIPQUAD(dest->vaddr),
+                          ntohs(dest->vport));
 
 		ret = __ip_vs_update_dest(svc, dest, ur);
 		if (ret)
@@ -743,6 +743,9 @@ static int ip_vs_add_dest(struct ip_vs_service *svc,
 
 		list_add(&dest->n_list, &svc->destinations);
 
+                /* call the update_service function of its scheduler */
+                svc->scheduler->update_service(svc);
+
 		write_unlock_bh(&__ip_vs_svc_lock);
 		return 0;
 	}
@@ -769,7 +772,10 @@ static int ip_vs_add_dest(struct ip_vs_service *svc,
 
 	list_add(&dest->n_list, &svc->destinations);
 
-	write_unlock_bh(&__ip_vs_svc_lock);
+        /* call the update_service function of its scheduler */
+        svc->scheduler->update_service(svc);
+
+        write_unlock_bh(&__ip_vs_svc_lock);
 
         LeaveFunction(2);
         
