@@ -1,7 +1,7 @@
 /*
  * IP_VS        ftp application module
  *
- * Version:	$Id: ip_vs_ftp.c,v 1.3 2001/01/10 07:42:53 wensong Exp $
+ * Version:	$Id: ip_vs_ftp.c,v 1.4 2001/02/23 09:15:05 wensong Exp $
  *
  * Authors:	Wensong Zhang <wensong@linuxvirtualserver.org>
  *		
@@ -127,10 +127,11 @@ static int ip_vs_ftp_get_addrport(char *data, char *data_limit,
 /*
  * Look at outgoing ftp packets to catch the response to a PASV command
  * from the server (inside-to-outside).
- * When we see one, we build a connection entry for the client address,
+ * When we see one, we build a connection entry with the client address,
  * client port 0 (unknown at the moment), the server address and the
  * server port.  Mark the current connection entry as a control channel
- * of the new entry. All this work just to keep data connection entry alive.
+ * of the new entry. All this work is just to make the data connection
+ * can be scheduled to the right server later.
  *
  * The outgoing packet should be something like
  *   "227 Entering Passive Mode (xxx,xxx,xxx,xxx,ppp,ppp)".
@@ -155,10 +156,9 @@ static int ip_vs_ftp_out(struct ip_vs_app *mapp,
                 skb = *skb_p;
                 iph = skb->nh.iph;
                 th = (struct tcphdr *)&(((char *)iph)[iph->ihl*4]);
-                data = (char *)&th[1];
+                data = (char *)th + (th->doff << 2);
                 data_limit = skb->h.raw + skb->len;
 
-
                 if (ip_vs_ftp_get_addrport(data, data_limit,
                                            SERVER_STRING,
                                            sizeof(SERVER_STRING)-1, ')',
@@ -181,10 +181,15 @@ static int ip_vs_ftp_out(struct ip_vs_app *mapp,
                                               cp->vaddr, port,
                                               from, port,
                                               IP_VS_CONN_F_NO_CPORT);
-
                         if (!n_cp)
                                 return 0;
+
+                        /* add its controller and bind its destination */
                         ip_vs_control_add(n_cp, cp);
+                        ip_vs_bind_dest(n_cp, cp->dest);
+                        
+                        /* increase dest's inactive connection counter */
+                        atomic_inc(&cp->dest->inactconns);
                 }
 
 		/*
@@ -243,7 +248,7 @@ static int ip_vs_ftp_in(struct ip_vs_app *mapp,
 	struct sk_buff *skb;
 	struct iphdr *iph;
 	struct tcphdr *th;
-	char *data, *data_limit;
+	char *data, *data_start, *data_limit;
 	char *start, *end;
 	__u32 to;
 	__u16 port;
@@ -255,14 +260,18 @@ static int ip_vs_ftp_in(struct ip_vs_app *mapp,
         skb = *skb_p;
 	iph = skb->nh.iph;
         th = (struct tcphdr *)&(((char *)iph)[iph->ihl*4]);
-        data = (char *)&th[1];
+
+        /* Since there may be OPTIONS in the TCP packet and the HLEN is
+           the length of the header in 32-bit multiples, it is accurate
+           to caculate data address by th+HLEN*4 */
+        data = data_start = (char *)th + (th->doff << 2);
         data_limit = skb->h.raw + skb->len;
 
 	while (data < data_limit) {
                 if (strnicmp(data, "PASV\r\n", 6) == 0) {
                         IP_VS_DBG(1-debug, "got PASV at %d of %d\n",
-                                  data-(char *)&th[1],
-                                  data_limit-(char *)&th[1]);
+                                  data - data_start,
+                                  data_limit - data_start);
                         cp->app_data = &ip_vs_ftp_pasv;
                         return 0;
                 }
@@ -276,7 +285,7 @@ static int ip_vs_ftp_in(struct ip_vs_app *mapp,
 	 * then create a new masquerading entry for the coming data
 	 * connection.
 	 */
-        data = (char *)&th[1];
+        data = data_start;
         data_limit = skb->h.raw + skb->len - 18;
 
         if (ip_vs_ftp_get_addrport(data, data_limit,
@@ -306,7 +315,13 @@ static int ip_vs_ftp_in(struct ip_vs_app *mapp,
                                       0);
                 if (!n_cp)
                         return 0;
+                
+                /* add its controller and bind its destination */
                 ip_vs_control_add(n_cp, cp);
+                ip_vs_bind_dest(n_cp, cp->dest);
+
+                /* increase dest's inactive connection counter */
+                atomic_inc(&cp->dest->inactconns);
         }
 
         /*
